<?php

try {
    require_once(dirname(__FILE__) . '/application.php');
    require_once(dirname(__FILE__) . '/setup.php');
    require_once(dirname(__FILE__) . '/lib/fix.php');

    class Surfacer_ExtendAddTags extends Surfacer
    {
        function buildSurface(& $Tpl, $root)
        {
            $this->listBlog($Tpl, $root);

            if ( $this->Post->isExists('certainly') ) {
                $Tpl->add(array('certainly', $root), array('certainly:checked' => ' checked="checked"'));
            } else {
                $Tpl->add(array('certainly', $root));
            }

            if ( $this->Post->isExists('preview') && !empty($this->Error['preview']) ) {
                $words = $this->business();

                if ( is_array($words) ) {
                    foreach ( $words as $word => $es ) {
                        foreach ( $es as $e ) {
                            $e['fulltext'] = preg_replace('@(' . $word . ')@', '<strong class="highlight">$1</strong>', $e['fulltext']);
                            $Tpl->add(array('entry:loop', 'word:loop', $root), $e);
                        }
                        $Tpl->add(array('word:loop', $root), array('word' => $word));
                    }
                } else {
                    $Tpl->add(array('notFound', $root));
                }
            }
        }

        function validate()
        {
            if ( !empty($_FILES['source']['error']) ) {
                $this->Error['file_upload'] = true;
            }

            if ( $this->Post->isNull('bid') ) $this->Error['blog_select'] = true;

            $this->detectStep();
        }

        function business()
        {
            @set_time_limit(0);

            $filetype = $_FILES['source']['name'];
            $raw = file_get_contents($_FILES['source']['tmp_name']);

            /**
             * detect convert encoding
             */
            if ( $enc = mb_detect_encoding($raw, 'UTF-8, EUC-JP, SJIS, EUCJP-win, SJIS-win') ) {
                $raw = mb_convert_encoding($raw, 'UTF-8', $enc);
            }

            $fixed = preg_replace('@,| |　@', '', $raw);
            $words = preg_split('@[\x0D\x0A|\x0D|\x0A/]@', $fixed);
            $words = array_unique(array_merge(array_diff($words, array(''))));

            $i = 1;
            $bid = $this->Post->get('bid');
            $DB = DB::singleton(SetupCommon::dsn());

            /**
             * config
             */
            $title_certainly = $this->Post->get('certainly');
            $frequency_threshold = intval($this->Post->get('threshold'));
            $ignore_entries = $this->Post->getArray('ignore');
            if ( $this->Post->isExists('preview') ) {
                $this->Error['preview'] = true;
            }

            /**
             * word rotation & detect add tags
             */


            foreach ( $words as $word ) {
                $Tag = SQL::newSelect('tag');
                $Tag->addSelect('tag_entry_id');
                $Tag->addWhereOpr('tag_name', $word);

                $SQL = SQL::newSelect('entry');
                $SQL->addLeftJoin('fulltext', 'fulltext_eid', 'entry_id');
                $SQL->addLeftJoin('tag', 'tag_entry_id', 'entry_id');

                $SQL->addSelect('entry_id');
                $SQL->addSelect('entry_title');
                $SQL->addSelect('fulltext_value');
                $SQL->addSelect('tag_sort', 'tag_max', null, 'max');
                $SQL->addSelect('tag_name');

                $SQL->addWhereOpr('fulltext_value', '%' . $word . '%', 'LIKE');
                $SQL->addWhereOpr('entry_blog_id', $bid);
                $SQL->addWhereNotIn('entry_id', $Tag, 'AND');

                $SQL->addGroup('entry_id');
                $q = $SQL->get(SetupCommon::dsn());

                $DB->query($q, 'fetch');

                while ( $e = $DB->fetch($q) ) {
                    //echo $i++.': '.$e['entry_title'].' # '.$e['tag_max'].'tags @'.$e['tag_name'].'<br />';

                    $insert = false;
                    $eid = $e['entry_id'];
                    $title = $e['entry_title'];
                    $text = $e['fulltext_value'];
                    $sort = $e['tag_max'] + 1;

                    // continue when "is not set eid" OR "found on ignores list"
                    if ( empty($eid) || array_search($eid . '@' . $word, $ignore_entries) !== false ) continue;

                    if ( $title_certainly == 'on' && strstr($title, $word) !== false ) {
                        $insert = true;
                    } elseif ( substr_count($text, $word) >= $frequency_threshold ) {
                        $insert = true;
                    }

                    if ( $insert === true ) {
                        if ( $this->Post->isExists('preview') ) {
                            $this->Error['preview'] = true;
                            $preview[$word][] = array(
                                'title' => $title,
                                'fulltext' => substr($text, 0, 512),
                                'word' => $word,
                                'eid' => $eid,
                            );
                        } else {
                            $SQL = SQL::newInsert('tag');
                            $SQL->addInsert('tag_name', $word);
                            $SQL->addInsert('tag_sort', $sort);
                            $SQL->addInsert('tag_entry_id', $eid);
                            $SQL->addInsert('tag_blog_id', $bid);
                            $DB->query($SQL->get(SetupCommon::dsn()), 'exec');
                        }
                    } else {

                    }
                }
            }

            $this->detectStep();

            if ( !empty($preview) ) {
                return $preview;
            }
        }
    }

    class Surfacer_ExtendAddNgram extends Surfacer
    {
        function buildSurface(& $Tpl, $root)
        {

        }

        function validate()
        {
            if ( $this->Post->isNull('ngram') ) {
                $this->Error['ngram_input'] = true;
            }

            $ngram = intval($this->Post->get('ngram'));

            if ( 1 < strlen($ngram) || 0 >= $ngram ) {
                $this->Error['ngram_invalid'] = true;
            }

            $this->detectStep();
        }

        function business()
        {
            $args = array('ngram' => $this->Post->get('ngram'));

            $Fix = new Fix_Ngram($args);

            if ( !$Fix->execute() ) $this->Error['execute'] = true;

            $this->detectStep();
        }
    }

    switch ( htmlentities($_GET['c'], ENT_QUOTES, 'UTF-8') ) {
        case 'tags' :
            $tplPath = dirname(__FILE__) . "/tpl/ext/add-tags.html";
            new Surfacer_ExtendAddTags($tplPath);
            break;
        case 'ngram' :
            $tplPath = dirname(__FILE__) . "/tpl/ext/add-ngram.html";
            new Surfacer_ExtendAddNgram($tplPath);
            break;
        default :
            header('Content-type: text/html; charset=UTF-8');
            die('<p>不正な命令です</p>');
            break;
    }

    App::checkException();
} catch ( Exception $e ) {
    App::showError($e);
}